Package: yuimaGUI
Type: Package
Title: A Graphical User Interface for the 'yuima' Package
Version: 1.4.0
Authors@R: c(person(given = c("YUIMA", "Project"),
                      family = "Team",
                      role = "aut"),
               person(given = "Emanuele",
                      family = "Guidotti",
                      role = "cre",
                      email = "emanuele.guidotti@usi.ch"))
Description: Provides a graphical user interface for the 'yuima' package.
License: GPL-2
Depends: R(>= 3.0.0)
Imports: DT (>= 0.2), shinyjs, shiny, shinydashboard, shinyBS, yuima,
        quantmod, sde, ggplot2, plotly, ghyp
RoxygenNote: 7.2.1
URL: https://yuimaproject.com
BugReports: https://github.com/yuimaproject/yuimaGUI/issues
NeedsCompilation: no
Packaged: 2026-03-12 11:13:47 UTC; eguidotti
Author: YUIMA Project Team [aut],
  Emanuele Guidotti [cre]
Maintainer: Emanuele Guidotti <emanuele.guidotti@usi.ch>
Repository: CRAN
Date/Publication: 2026-03-12 13:50:02 UTC
Built: R 4.6.0; ; 2026-04-30 18:22:34 UTC; unix
