# yuimaGUI: A Graphical User Interface for the yuima Package

[![](https://cranlogs.r-pkg.org/badges/grand-total/yuimaGUI?color=blue)](https://cran.r-project.org/package=yuimaGUI)

This R package provides a user-friendly interface for [yuima](https://github.com/yuimaproject/yuima). A user guide is available [here](https://storage.googleapis.com/yuimaproject/2019/03/manual_yuimagui_v1-02.pdf).

## Installation

Install the stable release from CRAN:

```R
install.packages("yuimaGUI")
```

## Run the interface

```R
library("yuimaGUI")
yuimaGUI()
```

## Acknowledgments

The Project has been funded up to 2010 by the Japan Science Technology (JST) Basic Research Programs PRESTO, Grants-in-Aid for Scientific Research No. 19340021. Presently, the YUIMA Project is supported by the Japan Science and Technology Agency CREST.

